package top.octopusyan;

import com.sun.org.slf4j.internal.Logger;
import com.sun.org.slf4j.internal.LoggerFactory;
import javafx.application.Application;
import javafx.application.Platform;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import top.octopusyan.common.http.HttpConfig;
import top.octopusyan.common.http.config.LogStrategy;
import top.octopusyan.common.http.request.RequestHandler;
import top.octopusyan.http.OkHttpClientConfig;
import top.octopusyan.utils.AlertUtil;
import top.octopusyan.utils.ApplicatonStore;

/**
 * @author : octopus yan
 * @email : octopus_yan@foxmail.com
 * @description : YanFrp Application
 * @create : 2022-3-29 15:00
 */
public class YanFrpApplication extends Application {
    private static final Logger LOGGER = LoggerFactory.getLogger(YanFrpApplication.class);

    @Override
    public void init() throws Exception {
        super.init();
        LOGGER.debug("init...");

        // 网络请求设置
        HttpConfig.with(OkHttpClientConfig.httpClient())
                // 请求服务地址
                .serverPath("https://frp.octopusyan.top")
                // 是否打印日志
                .setLogEnabled(false)
                // 设置日志打印策略
                .setLogStrategy(new LogStrategy())
                // 设置请求处理策略
                .setHandler(new RequestHandler())
                // 设置请求重试次数
                .setRetryCount(3)
                // 启用配置
                .into();
    }

    @Override
    public void start(Stage stage) {
        LOGGER.debug("start...");

        //  TODO 全局异常处理... (emm有点草率，先这样了。。)
        Thread.setDefaultUncaughtExceptionHandler((t, e) -> Platform.runLater(() -> showErrorDialog(t, e)));
        Thread.currentThread().setUncaughtExceptionHandler(this::showErrorDialog);

        // 初始化弹窗
        AlertUtil.initOwner(stage);
        // 初始化应用数据
        ApplicatonStore.setRegisterSuccess(false);

        try {
            stage.initStyle(StageStyle.TRANSPARENT);
            StackPane root = new FXMLLoader(this.getClass().getResource("/fxml/login.fxml")).load();//底层面板
            Scene scene = new Scene(
                    root,
                    root.getPrefWidth() + 20,
                    root.getPrefHeight() + 20,
                    Color.TRANSPARENT
            );
            scene.getStylesheets().addAll(getClass().getResource("/css/root.css").toExternalForm());
            stage.setScene(scene);
            stage.show();

        } catch (Throwable t) {
            showErrorDialog(Thread.currentThread(), t);
        }

        LOGGER.debug("start end...");
    }

    private void showErrorDialog(Thread t, Throwable e) {
        e.printStackTrace();
        AlertUtil.exceptionAlert(new Exception(e)).show();
    }

    @Override
    public void stop() throws Exception {
        super.stop();
        LOGGER.debug("stop...");
    }

    public static void main(String[] args) {
        Application.launch(YanFrpApplication.class, args);
    }
}
